==> relations-0.0.0-src.zip: relations/src/net/morilib/db/schema/FileSqlSchema.java <==
/*
 * Copyright 2013 Yuichiro Moriguchi
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this Fichier except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.morilib.db.schema;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.Reader;
import java.sql.SQLException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import net.morilib.db.fichier.Fichier;
import net.morilib.db.fichier.FabriqueDeFichier;
import net.morilib.db.map.CSVDataMapper;
import net.morilib.db.map.SqlDataMapper;
import net.morilib.db.misc.ErrorBundle;
import net.morilib.db.misc.NullBoolean;
import net.morilib.db.relations.NamedRelation;
import net.morilib.db.relations.RelationTuple;
import net.morilib.db.sql.DbSqlLexer;
import net.morilib.db.sql.DbSqlParser;
import net.morilib.db.sqlcs.ddl.SqlCreateTable;

public class FileSqlSchema implements SqlSchema {

	private static final SimpleDateFormat FMT =
			new SimpleDateFormat("'.bk'yyyyMMddHHmmssSSS");

	private FabriqueDeFichier fabrique;
	private Fichier base;

	public FileSqlSchema(FabriqueDeFichier f, Fichier s) {
		fabrique = f;
		base = s;
	}

	protected Fichier getFile(
			String name) throws IOException, SQLException {
		String t = name.toLowerCase();
		Fichier f;

		if(!isTable(name.toLowerCase())) {
			throw ErrorBundle.getDefault(10015, name);
		} else if((f = fabrique().newInstance(
				base, t + ".csv")).isFile()) {
			return f;
		} else {
			f = fabrique().newInstance(base, t + ".csv");
			f.createNewFile();
			return f;
		}
	}

	protected Reader getReader(
			String name) throws IOException, SQLException {
		return fabrique().newInstance(
				base, name.toLowerCase()).openReader();
	}

	protected Reader getTableReader(
			String name) throws IOException, SQLException {
		return getFile(name).openReader();
	}

	protected PrintWriter getWriter(
			String name) throws IOException, SQLException {
		Fichier f;

		f = fabrique().newInstance(base, name.toLowerCase());
		return new PrintWriter(new BufferedWriter(f.openWriter()));
	}

	protected PrintWriter getTableWriter(
			String name) throws IOException, SQLException {
		return new PrintWriter(new BufferedWriter(
				getFile(name).openWriter()));
	}

	@Override
	public SqlCreateTable getCreateTable(
			String name) throws IOException, SQLException {
		Reader rd = getReader(name + ".ddl");
		DbSqlLexer l;
		Object o;

		try {
			l = new DbSqlLexer(rd);
			o = new DbSqlParser().parse(l);
			if(o instanceof SqlCreateTable) {
				return (SqlCreateTable)o;
			} else {
				throw ErrorBundle.getDefault(10041);
			}
		} finally {
			rd.close();
		}
	}

	@Override
	public boolean isTable(
			String name) throws IOException, SQLException {
		return fabrique().newInstance(
				base, name + ".ddl").isFile();
	}

	@Override
	public void putCreateTable(String name,
			SqlCreateTable table) throws IOException, SQLException {
		PrintWriter w = null;

		if(isTable(name)) {
			throw ErrorBundle.getDefault(10016, name);
		} else {
			try {
				w = getWriter(name + ".ddl");
				w.write(table.toString());
			} finally {
				if(w != null)  w.close();
			}
		}
	}

	@Override
	public void alterCreateTable(String name,
			SqlCreateTable table) throws IOException, SQLException {
		PrintWriter w = null;

		if(!isTable(name)) {
			throw ErrorBundle.getDefault(10015, name);
		} else {
			try {
				w = getWriter(name + ".ddl");
				w.write(table.toString());
			} finally {
				if(w != null)  w.close();
			}
		}
	}

	@Override
	public NamedRelation readRelation(String t,
			String as) throws IOException, SQLException {
		Fichier f = getFile(t);
		SqlDataMapper m;

		if(f.getName().endsWith(".csv")) {
			m = new CSVDataMapper();
		} else {
			throw ErrorBundle.getDefault(10042);
		}
		return m.read(t, as, getCreateTable(t), getTableReader(t));
	}

	@Override
	public void writeRelation(String name,
			Collection<RelationTuple> z
			) throws IOException, SQLException {
		SqlCreateTable c = getCreateTable(name);
		Fichier f = getFile(name), t = null;
		char[] b = new char[1024];
		PrintWriter w = null;
		SqlDataMapper m;
		Reader r = null;
		int j;

		if(f.getName().endsWith(".csv")) {
			m = new CSVDataMapper();
		} else {
			throw ErrorBundle.getDefault(10042);
		}

		try {
			t = fabrique().createTempFile("pre-", ".tmp");
			w = new PrintWriter(new BufferedWriter(t.openWriter()));
			m.write(name, c, w, z);
			w.close();  w = null;

			r = new BufferedReader(t.openReader());
			w = getTableWriter(name);
			while((j = r.read(b)) >= 0)  w.write(b, 0, j);
		} finally {
			if(w != null)  w.close();
			if(r != null)  r.close();
			if(t != null)  t.delete();
		}
	}

	@Override
	public Collection<String> getTableNames() {
		List<String> r = new ArrayList<String>();
		Fichier[] l = base.listFichiers();
		String s;

		for(Fichier f : l) {
			s = f.getName();
			if(s.endsWith(".ddl")) {
				r.add(s.substring(0, s.length() - 4));
			}
		}
		return r;
	}

	@Override
	public void truncateTable(
			String name) throws IOException, SQLException {
		Fichier f, g;

		if(isTable(name)) {
			f = getFile(name);
			g = fabrique().newInstance(base,
					f.getName() + FMT.format(new java.util.Date()));
			f.renameTo(g);
		} else {
			throw ErrorBundle.getDefault(10015, name);
		}
	}

	@Override
	public void removeCreateTable(
			String name) throws IOException, SQLException {
		Fichier f, g;
		String s;

		if(isTable(name)) {
			s = FMT.format(new java.util.Date());
			f = getFile(name);
			g = fabrique().newInstance(base, f.getName() + s);
			f.renameTo(g);

			f = fabrique().newInstance(
					base, name.toLowerCase() + ".ddl");
			g = fabrique().newInstance(base, f.getName() + s);
			f.renameTo(g);
		} else {
			throw ErrorBundle.getDefault(10015, name);
		}
	}

	/* (non-Javadoc)
	 * @see net.morilib.db.schema.SqlSchema#fork()
	 */
	@Override
	public SqlSchema fork() {
		return new FileSqlSchema(fabrique, base);
	}

	@Override
	public NullBoolean isReadonly() {
		return NullBoolean.FALSE;
	}

	@Override
	public NullBoolean usesLocalFiles() {
		return NullBoolean.TRUE;
	}

	@Override
	public NullBoolean usesLocalFilePerTable() {
		return NullBoolean.TRUE;
	}

	@Override
	public FabriqueDeFichier fabrique() {
		return fabrique;
	}

}

==> relations-0.1.0-src.zip: relations/src/net/morilib/db/schema/FileSqlSchema.java <==
/*
 * Copyright 2013 Yuichiro Moriguchi
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this Fichier except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.morilib.db.schema;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.Reader;
import java.sql.SQLException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import net.morilib.db.fichier.Fichier;
import net.morilib.db.fichier.FabriqueDeFichier;
import net.morilib.db.map.CSVDataMapper;
import net.morilib.db.map.SqlDataMapper;
import net.morilib.db.misc.ErrorBundle;
import net.morilib.db.misc.NullBoolean;
import net.morilib.db.relations.NamedRelation;
import net.morilib.db.relations.Relation;
import net.morilib.db.relations.RelationTuple;
import net.morilib.db.relations.TableRenameRelation;
import net.morilib.db.sql.DbSqlLexer;
import net.morilib.db.sql.DbSqlParser;
import net.morilib.db.sqlcs.ddl.SqlCreateTable;

public class FileSqlSchema implements SqlSchema {

	private static final SimpleDateFormat FMT =
			new SimpleDateFormat("'.bk'yyyyMMddHHmmssSSS");

	private FabriqueDeFichier fabrique;
	private Fichier base;
	private Map<String, Relation> rels;

	public FileSqlSchema(FabriqueDeFichier f, Fichier s) {
		rels = new HashMap<String, Relation>();
		fabrique = f;
		base = s;
	}

	protected Fichier getFile(
			String name) throws IOException, SQLException {
		String t = name.toLowerCase();
		Fichier f;

		if(!isTable(name.toLowerCase())) {
			throw ErrorBundle.getDefault(10015, name);
		} else if((f = fabrique().newInstance(
				base, t + ".csv")).isFile()) {
			return f;
		} else {
			f = fabrique().newInstance(base, t + ".csv");
			f.createNewFile();
			return f;
		}
	}

	protected Reader getReader(
			String name) throws IOException, SQLException {
		return fabrique().newInstance(
				base, name.toLowerCase()).openReader();
	}

	protected Reader getTableReader(
			String name) throws IOException, SQLException {
		return getFile(name).openReader();
	}

	protected PrintWriter getWriter(
			String name) throws IOException, SQLException {
		Fichier f;

		f = fabrique().newInstance(base, name.toLowerCase());
		return new PrintWriter(new BufferedWriter(f.openWriter()));
	}

	protected PrintWriter getTableWriter(
			String name) throws IOException, SQLException {
		return new PrintWriter(new BufferedWriter(
				getFile(name).openWriter()));
	}

	@Override
	public SqlCreateTable getCreateTable(
			String name) throws IOException, SQLException {
		Reader r = null;
		DbSqlLexer l;
		Object o;

		try {
			r = getReader(name + ".ddl");
			l = new DbSqlLexer(r);
			o = new DbSqlParser().parse(l);
			if(o instanceof SqlCreateTable) {
				return (SqlCreateTable)o;
			} else {
				throw ErrorBundle.getDefault(10041);
			}
		} catch(FileNotFoundException e) {
			throw ErrorBundle.getDefault(10015, name);
		} finally {
			if(r != null)  r.close();
		}
	}

	@Override
	public boolean isTable(
			String name) throws IOException, SQLException {
		return fabrique().newInstance(
				base, name.toLowerCase() + ".ddl").isFile();
	}

	@Override
	public void putCreateTable(String name,
			SqlCreateTable table) throws IOException, SQLException {
		PrintWriter w = null;

		if(isTable(name)) {
			throw ErrorBundle.getDefault(10016, name);
		} else {
			try {
				w = getWriter(name + ".ddl");
				w.write(table.toString());
			} finally {
				if(w != null)  w.close();
			}
		}
	}

	@Override
	public void alterCreateTable(String name,
			SqlCreateTable table) throws IOException, SQLException {
		PrintWriter w = null;

		if(!isTable(name)) {
			throw ErrorBundle.getDefault(10015, name);
		} else {
			try {
				w = getWriter(name + ".ddl");
				w.write(table.toString());
			} finally {
				if(w != null)  w.close();
			}
		}
	}

	@Override
	public NamedRelation readRelation(String t,
			String as) throws IOException, SQLException {
		SqlDataMapper m;

		if(rels != null && rels.containsKey(t.toUpperCase())) {
			return new TableRenameRelation(
					rels.get(t.toUpperCase()),
					(as != null ? as : t).toUpperCase());
		} else if(getFile(t).getName().endsWith(".csv")) {
			m = new CSVDataMapper();
		} else {
			throw ErrorBundle.getDefault(10042);
		}
		return m.read(t, as, getCreateTable(t), getTableReader(t));
	}

	@Override
	public void writeRelation(String name,
			Collection<RelationTuple> z
			) throws IOException, SQLException {
		SqlCreateTable c = getCreateTable(name);
		Fichier f = getFile(name), t = null;
		char[] b = new char[1024];
		PrintWriter w = null;
		SqlDataMapper m;
		Reader r = null;
		int j;

		if(f.getName().endsWith(".csv")) {
			m = new CSVDataMapper();
		} else {
			throw ErrorBundle.getDefault(10042);
		}

		try {
			t = fabrique().createTempFile("pre-", ".tmp");
			w = new PrintWriter(new BufferedWriter(t.openWriter()));
			m.write(name, c, w, z);
			w.close();  w = null;

			r = new BufferedReader(t.openReader());
			w = getTableWriter(name);
			while((j = r.read(b)) >= 0)  w.write(b, 0, j);
		} finally {
			if(w != null)  w.close();
			if(r != null)  r.close();
			if(t != null)  t.delete();
		}
	}

	@Override
	public Collection<String> getTableNames() {
		List<String> r = new ArrayList<String>();
		Fichier[] l = base.listFichiers();
		String s;

		for(Fichier f : l) {
			s = f.getName();
			if(s.endsWith(".ddl")) {
				r.add(s.substring(0, s.length() - 4));
			}
		}
		return r;
	}

	@Override
	public void truncateTable(
			String name) throws IOException, SQLException {
		Fichier f, g;

		if(isTable(name)) {
			f = getFile(name);
			g = fabrique().newInstance(base,
					f.getName() + FMT.format(new java.util.Date()));
			f.renameTo(g);
		} else {
			throw ErrorBundle.getDefault(10015, name);
		}
	}

	@Override
	public void removeCreateTable(
			String name) throws IOException, SQLException {
		Fichier f, g;
		String s;

		if(isTable(name)) {
			s = FMT.format(new java.util.Date());
			f = getFile(name);
			g = fabrique().newInstance(base, f.getName() + s);
			f.renameTo(g);

			f = fabrique().newInstance(
					base, name.toLowerCase() + ".ddl");
			g = fabrique().newInstance(base, f.getName() + s);
			f.renameTo(g);
		} else {
			throw ErrorBundle.getDefault(10015, name);
		}
	}

	/* (non-Javadoc)
	 * @see net.morilib.db.schema.SqlSchema#fork()
	 */
	@Override
	public SqlSchema fork() {
		return new FileSqlSchema(fabrique, base);
	}

	@Override
	public NullBoolean isReadonly() {
		return NullBoolean.FALSE;
	}

	@Override
	public NullBoolean usesLocalFiles() {
		return NullBoolean.TRUE;
	}

	@Override
	public NullBoolean usesLocalFilePerTable() {
		return NullBoolean.TRUE;
	}

	@Override
	public FabriqueDeFichier fabrique() {
		return fabrique;
	}

	/* (non-Javadoc)
	 * @see net.morilib.db.schema.SqlSchema#isAutoCommit()
	 */
	@Override
	public boolean isAutoCommit() {
		return true;
	}

	/* (non-Javadoc)
	 * @see net.morilib.db.schema.SqlSchema#bindSchema(java.lang.String, net.morilib.db.relations.Relation)
	 */
	@Override
	public void bindSchema(String name, Relation r) {
		rels.put(name.toUpperCase(), r);
	}

	/* (non-Javadoc)
	 * @see net.morilib.db.schema.SqlSchema#isLocked(java.lang.String)
	 */
	@Override
	public boolean isLocked(String name) {
		Fichier f;

		f = fabrique.newInstance(base, name + ".lock");
		return f.isFile();
	}

	/* (non-Javadoc)
	 * @see net.morilib.db.schema.SqlSchema#isLocked(java.lang.String)
	 */
	@Override
	public boolean isUsed(String name) {
		Fichier f;

		f = fabrique.newInstance(base, name + ".lock");
		return f.isFile();
	}

	/* (non-Javadoc)
	 * @see net.morilib.db.schema.SqlSchema#lock(java.lang.String)
	 */
	@Override
	public void lock(String name) throws IOException {
		Fichier f;

		f = fabrique.newInstance(base, name + ".lock");
		f.createNewFile();
	}

	/* (non-Javadoc)
	 * @see net.morilib.db.schema.SqlSchema#unlock(java.lang.String)
	 */
	@Override
	public void unlock(String name) throws IOException {
		Fichier f;

		f = fabrique.newInstance(base, name + ".lock");
		f.delete();
	}

}
